@@ -11,9 +11,10 @@ from sqlalchemy.orm import relation, backref
 from sqlalchemy.ext.hybrid import Comparator, hybrid_property
 from sqlalchemy.exc import OperationalError
 
-from flexget import db_schema
+from flexget import db_schema, options, plugin
 from flexget.config_schema import one_or_more
 from flexget.event import event
+from flexget.manager import Session
 from flexget.utils import qualities
 from flexget.utils.log import log_once
 from flexget.utils.titles import SeriesParser, ParseWarning, ID_TYPES
@@ -21,9 +22,6 @@ from flexget.utils.sqlalchemy_utils import (table_columns, table_exists, drop_ta
                                             create_index)
 from flexget.utils.tools import merge_dict_from_to, parse_timedelta
 from flexget.utils.database import quality_property
-from flexget.manager import Session
-from flexget.plugin import (register_plugin, register_parser_option, get_plugin_by_name, DependencyError, priority,
-                            PluginError)
 
 SCHEMA_VER = 11
 
@@ -164,18 +162,40 @@ def db_cleanup(session):
         log.verbose('Removed %d series without episodes.', result)
 
 
-@event('manager.startup')
+@event('manager.lock-acquired')
 def repair(manager):
-    """Perform database repairing and upgrading at startup."""
+    # Perform database repairing and upgrading at startup.
     if not manager.persist.get('series_repaired', False):
         session = Session()
-        # For some reason at least I have some releases in database which don't belong to any episode.
-        for release in session.query(Release).filter(Release.episode == None).all():
-            log.info('Purging orphan release %s from database', release.title)
-            session.delete(release)
-        session.commit()
+        try:
+            # For some reason at least I have some releases in database which don't belong to any episode.
+            for release in session.query(Release).filter(Release.episode == None).all():
+                log.info('Purging orphan release %s from database', release.title)
+                session.delete(release)
+            session.commit()
+        finally:
+            session.close()
         manager.persist['series_repaired'] = True
 
+    # Run clean_series the first time we get a database lock, since we won't have had one the first time the config
+    # got loaded.
+    clean_series(manager)
+
+
+@event('manager.config-loaded')
+def clean_series(manager):
+    # Unmark series from tasks which have been deleted.
+    if not manager.has_lock:
+        return
+    session = Session()
+    try:
+        deleted = (session.query(SeriesTask).filter(not_(SeriesTask.name.in_(manager.tasks))).
+                   delete(synchronize_session=False))
+        if deleted:
+            session.commit()
+    finally:
+        session.close()
+
 
 TRANSLATE_MAP = {ord(u'&'): u' and '}
 for char in u'\'\\':
@@ -362,176 +382,176 @@ class SeriesTask(Base):
         self.name = name
 
 
-class SeriesDatabase(object):
+def get_latest_episode(series):
+    """Return latest known identifier in dict (season, episode, name) for series name"""
+    session = Session.object_session(series)
+    episode = session.query(Episode).join(Episode.series).\
+        filter(Series.id == series.id).\
+        filter(Episode.season != None).\
+        order_by(desc(Episode.season)).\
+        order_by(desc(Episode.number)).first()
+    if not episode:
+        # log.trace('get_latest_info: no info available for %s', name)
+        return False
+    # log.trace('get_latest_info, series: %s season: %s episode: %s' % \
+    #    (name, episode.season, episode.number))
+    return episode
 
-    """Provides API to series database"""
 
-    def get_latest_episode(self, series):
-        """Return latest known identifier in dict (season, episode, name) for series name"""
-        session = Session.object_session(series)
-        episode = session.query(Episode).join(Episode.series).\
-            filter(Series.id == series.id).\
-            filter(Episode.season != None).\
-            order_by(desc(Episode.season)).\
-            order_by(desc(Episode.number)).first()
-        if not episode:
-            # log.trace('get_latest_info: no info available for %s', name)
-            return False
-        # log.trace('get_latest_info, series: %s season: %s episode: %s' % \
-        #    (name, episode.season, episode.number))
-        return episode
+def auto_identified_by(series):
+    """
+    Determine if series `name` should be considered identified by episode or id format
 
-    def auto_identified_by(self, series):
-        """
-        Determine if series `name` should be considered identified by episode or id format
+    Returns 'ep', 'sequence', 'date' or 'id' if enough history is present to identify the series' id type.
+    Returns 'auto' if there is not enough history to determine the format yet
+    """
 
-        Returns 'ep', 'sequence', 'date' or 'id' if enough history is present to identify the series' id type.
-        Returns 'auto' if there is not enough history to determine the format yet
-        """
-
-        session = Session.object_session(series)
-        type_totals = dict(session.query(Episode.identified_by, func.count(Episode.identified_by)).join(Episode.series).
-                           filter(Series.id == series.id).group_by(Episode.identified_by).all())
-        # Remove None and specials from the dict,
-        # we are only considering episodes that we know the type of (parsed with new parser)
-        type_totals.pop(None, None)
-        type_totals.pop('special', None)
-        if not type_totals:
-            return 'auto'
-        log.debug('%s episode type totals: %r', series.name, type_totals)
-        # Find total number of parsed episodes
-        total = sum(type_totals.itervalues())
-        # See which type has the most
-        best = max(type_totals, key=lambda x: type_totals[x])
-
-        # Ep mode locks in faster than the rest. At 2 seen episodes.
-        if type_totals.get('ep', 0) >= 2 and type_totals['ep'] > total / 3:
-            log.info('identified_by has locked in to type `ep` for %s', series.name)
-            return 'ep'
-        # If we have over 3 episodes all of the same type, lock in
-        if len(type_totals) == 1 and total >= 3:
-            return best
-        # Otherwise wait until 5 episodes to lock in
-        if total >= 5:
-            log.info('identified_by has locked in to type `%s` for %s', best, series.name)
-            return best
-        log.verbose('identified by is currently on `auto` for %s. '
-                    'Multiple id types may be accepted until it locks in on the appropriate type.', series.name)
+    session = Session.object_session(series)
+    type_totals = dict(session.query(Episode.identified_by, func.count(Episode.identified_by)).join(Episode.series).
+                       filter(Series.id == series.id).group_by(Episode.identified_by).all())
+    # Remove None and specials from the dict,
+    # we are only considering episodes that we know the type of (parsed with new parser)
+    type_totals.pop(None, None)
+    type_totals.pop('special', None)
+    if not type_totals:
         return 'auto'
+    log.debug('%s episode type totals: %r', series.name, type_totals)
+    # Find total number of parsed episodes
+    total = sum(type_totals.itervalues())
+    # See which type has the most
+    best = max(type_totals, key=lambda x: type_totals[x])
 
-    def get_latest_download(self, series):
-        """
-        :param Series series: SQLAlchemy session
-        :return: Instance of Episode or None if not found.
-        """
-        session = Session.object_session(series)
-        downloaded = session.query(Episode).join(Episode.releases, Episode.series).\
-            filter(Series.id == series.id).\
-            filter(Release.downloaded == True)
-        if series.identified_by and series.identified_by != 'auto':
-            downloaded = downloaded.filter(Episode.identified_by == series.identified_by)
-        if series.identified_by in ['ep', 'sequence']:
-            latest_download = downloaded.order_by(desc(Episode.season), desc(Episode.number)).first()
-        elif series.identified_by == 'date':
-            latest_download = downloaded.order_by(desc(Episode.identifier)).first()
-        else:
-            latest_download = downloaded.order_by(desc(Episode.first_seen)).first()
+    # Ep mode locks in faster than the rest. At 2 seen episodes.
+    if type_totals.get('ep', 0) >= 2 and type_totals['ep'] > total / 3:
+        log.info('identified_by has locked in to type `ep` for %s', series.name)
+        return 'ep'
+    # If we have over 3 episodes all of the same type, lock in
+    if len(type_totals) == 1 and total >= 3:
+        return best
+    # Otherwise wait until 5 episodes to lock in
+    if total >= 5:
+        log.info('identified_by has locked in to type `%s` for %s', best, series.name)
+        return best
+    log.verbose('identified by is currently on `auto` for %s. '
+                'Multiple id types may be accepted until it locks in on the appropriate type.', series.name)
+    return 'auto'
 
-        if not latest_download:
-            log.debug('get_latest_download returning None, no downloaded episodes found for: %s', series.name)
-            return
 
-        return latest_download
+def get_latest_download(series):
+    """
+    :param Series series: SQLAlchemy session
+    :return: Instance of Episode or None if not found.
+    """
+    session = Session.object_session(series)
+    downloaded = session.query(Episode).join(Episode.releases, Episode.series).\
+        filter(Series.id == series.id).\
+        filter(Release.downloaded == True)
+    if series.identified_by and series.identified_by != 'auto':
+        downloaded = downloaded.filter(Episode.identified_by == series.identified_by)
+    if series.identified_by in ['ep', 'sequence']:
+        latest_download = downloaded.order_by(desc(Episode.season), desc(Episode.number)).first()
+    elif series.identified_by == 'date':
+        latest_download = downloaded.order_by(desc(Episode.identifier)).first()
+    else:
+        latest_download = downloaded.order_by(desc(Episode.first_seen)).first()
 
-    def new_eps_after(self, since_ep):
-        """
-        :param since_ep: Episode instance
-        :return: Number of episodes since then
-        """
-        session = Session.object_session(since_ep)
-        series = since_ep.series
-        series_eps = session.query(Episode).join(Episode.series).\
-            filter(Series.id == series.id)
-        if series.identified_by == 'ep':
-            if since_ep.season is None or since_ep.number is None:
-                log.debug('new_eps_after for %s falling back to timestamp because latest dl in non-ep format' %
-                          series.name)
-                return series_eps.filter(Episode.first_seen > since_ep.first_seen).count()
-            return series_eps.filter((Episode.identified_by == 'ep') &
-                                     (((Episode.season == since_ep.season) & Episode.number > since_ep.number) |
-                                      Episode.season > since_ep.season)).count()
-        elif series.identified_by == 'seq':
-            return series_eps.filter(Episode.number > since_ep.number).count()
-        elif series.identified_by == 'id':
+    if not latest_download:
+        log.debug('get_latest_download returning None, no downloaded episodes found for: %s', series.name)
+        return
+
+    return latest_download
+
+
+def new_eps_after(since_ep):
+    """
+    :param since_ep: Episode instance
+    :return: Number of episodes since then
+    """
+    session = Session.object_session(since_ep)
+    series = since_ep.series
+    series_eps = session.query(Episode).join(Episode.series).\
+        filter(Series.id == series.id)
+    if series.identified_by == 'ep':
+        if since_ep.season is None or since_ep.number is None:
+            log.debug('new_eps_after for %s falling back to timestamp because latest dl in non-ep format' %
+                      series.name)
             return series_eps.filter(Episode.first_seen > since_ep.first_seen).count()
-        else:
-            log.debug('unsupported identified_by %s', series.identified_by)
-            return 0
+        return series_eps.filter((Episode.identified_by == 'ep') &
+                                 (((Episode.season == since_ep.season) & Episode.number > since_ep.number) |
+                                  Episode.season > since_ep.season)).count()
+    elif series.identified_by == 'seq':
+        return series_eps.filter(Episode.number > since_ep.number).count()
+    elif series.identified_by == 'id':
+        return series_eps.filter(Episode.first_seen > since_ep.first_seen).count()
+    else:
+        log.debug('unsupported identified_by %s', series.identified_by)
+        return 0
 
-    def store(self, session, parser, series=None):
-        """
-        Push series information into database. Returns added/existing release.
 
-        :param session: Database session to use
-        :param parser: parser for release that should be added to database
-        :param series: Series in database to add release to. Will be looked up if not provided.
-        :return: List of Releases
-        """
+def store_parser(session, parser, series=None):
+    """
+    Push series information into database. Returns added/existing release.
+
+    :param session: Database session to use
+    :param parser: parser for release that should be added to database
+    :param series: Series in database to add release to. Will be looked up if not provided.
+    :return: List of Releases
+    """
+    if not series:
+        # if series does not exist in database, add new
+        series = session.query(Series).\
+            filter(Series.name == parser.name).\
+            filter(Series.id != None).first()
         if not series:
-            # if series does not exist in database, add new
-            series = session.query(Series).\
-                filter(Series.name == parser.name).\
-                filter(Series.id != None).first()
-            if not series:
-                log.debug('adding series %s into db', parser.name)
-                series = Series()
-                series.name = parser.name
-                session.add(series)
-                log.debug('-> added %s' % series)
+            log.debug('adding series %s into db', parser.name)
+            series = Series()
+            series.name = parser.name
+            session.add(series)
+            log.debug('-> added %s' % series)
 
-        releases = []
-        for ix, identifier in enumerate(parser.identifiers):
-            # if episode does not exist in series, add new
-            episode = session.query(Episode).filter(Episode.series_id == series.id).\
-                filter(Episode.identifier == identifier).\
-                filter(Episode.series_id != None).first()
-            if not episode:
-                log.debug('adding episode %s into series %s', identifier, parser.name)
-                episode = Episode()
-                episode.identifier = identifier
-                episode.identified_by = parser.id_type
-                # if episodic format
-                if parser.id_type == 'ep':
-                    episode.season = parser.season
-                    episode.number = parser.episode + ix
-                elif parser.id_type == 'sequence':
-                    episode.season = 0
-                    episode.number = parser.id + ix
-                series.episodes.append(episode)  # pylint:disable=E1103
-                log.debug('-> added %s' % episode)
+    releases = []
+    for ix, identifier in enumerate(parser.identifiers):
+        # if episode does not exist in series, add new
+        episode = session.query(Episode).filter(Episode.series_id == series.id).\
+            filter(Episode.identifier == identifier).\
+            filter(Episode.series_id != None).first()
+        if not episode:
+            log.debug('adding episode %s into series %s', identifier, parser.name)
+            episode = Episode()
+            episode.identifier = identifier
+            episode.identified_by = parser.id_type
+            # if episodic format
+            if parser.id_type == 'ep':
+                episode.season = parser.season
+                episode.number = parser.episode + ix
+            elif parser.id_type == 'sequence':
+                episode.season = 0
+                episode.number = parser.id + ix
+            series.episodes.append(episode)  # pylint:disable=E1103
+            log.debug('-> added %s' % episode)
 
-            # if release does not exists in episode, add new
-            #
-            # NOTE:
-            #
-            # filter(Release.episode_id != None) fixes weird bug where release had/has been added
-            # to database but doesn't have episode_id, this causes all kinds of havoc with the plugin.
-            # perhaps a bug in sqlalchemy?
-            release = session.query(Release).filter(Release.episode_id == episode.id).\
-                filter(Release.title == parser.data).\
-                filter(Release.quality == parser.quality).\
-                filter(Release.proper_count == parser.proper_count).\
-                filter(Release.episode_id != None).first()
-            if not release:
-                log.debug('adding release %s into episode', parser)
-                release = Release()
-                release.quality = parser.quality
-                release.proper_count = parser.proper_count
-                release.title = parser.data
-                episode.releases.append(release)  # pylint:disable=E1103
-                log.debug('-> added %s' % release)
-            releases.append(release)
-        return releases
+        # if release does not exists in episode, add new
+        #
+        # NOTE:
+        #
+        # filter(Release.episode_id != None) fixes weird bug where release had/has been added
+        # to database but doesn't have episode_id, this causes all kinds of havoc with the plugin.
+        # perhaps a bug in sqlalchemy?
+        release = session.query(Release).filter(Release.episode_id == episode.id).\
+            filter(Release.title == parser.data).\
+            filter(Release.quality == parser.quality).\
+            filter(Release.proper_count == parser.proper_count).\
+            filter(Release.episode_id != None).first()
+        if not release:
+            log.debug('adding release %s into episode', parser)
+            release = Release()
+            release.quality = parser.quality
+            release.proper_count = parser.proper_count
+            release.title = parser.data
+            episode.releases.append(release)  # pylint:disable=E1103
+            log.debug('-> added %s' % release)
+        releases.append(release)
+    return releases
 
 
 def set_series_begin(series, ep_id):
@@ -588,32 +608,38 @@ def set_series_begin(series, ep_id):
 def forget_series(name):
     """Remove a whole series `name` from database."""
     session = Session()
-    series = session.query(Series).filter(Series.name == name).all()
-    if series:
-        for s in series:
-            session.delete(s)
-        session.commit()
-        log.debug('Removed series %s from database.', name)
-    else:
-        raise ValueError('Unknown series %s' % name)
+    try:
+        series = session.query(Series).filter(Series.name == name).all()
+        if series:
+            for s in series:
+                session.delete(s)
+            session.commit()
+            log.debug('Removed series %s from database.', name)
+        else:
+            raise ValueError('Unknown series %s' % name)
+    finally:
+        session.close()
 
 
 def forget_series_episode(name, identifier):
     """Remove all episodes by `identifier` from series `name` from database."""
     session = Session()
-    series = session.query(Series).filter(Series.name == name).first()
-    if series:
-        episode = session.query(Episode).filter(Episode.identifier == identifier).\
-            filter(Episode.series_id == series.id).first()
-        if episode:
-            series.identified_by = ''  # reset identified_by flag so that it will be recalculated
-            session.delete(episode)
-            session.commit()
-            log.debug('Episode %s from series %s removed from database.', identifier, name)
+    try:
+        series = session.query(Series).filter(Series.name == name).first()
+        if series:
+            episode = session.query(Episode).filter(Episode.identifier == identifier).\
+                filter(Episode.series_id == series.id).first()
+            if episode:
+                series.identified_by = ''  # reset identified_by flag so that it will be recalculated
+                session.delete(episode)
+                session.commit()
+                log.debug('Episode %s from series %s removed from database.', identifier, name)
+            else:
+                raise ValueError('Unknown identifier %s for series %s' % (identifier, name.capitalize()))
         else:
-            raise ValueError('Unknown identifier %s for series %s' % (identifier, name.capitalize()))
-    else:
-        raise ValueError('Unknown series %s' % name)
+            raise ValueError('Unknown series %s' % name)
+    finally:
+        session.close()
 
 
 def populate_entry_fields(entry, parser):
@@ -642,7 +668,7 @@ def populate_entry_fields(entry, parser):
 class FilterSeriesBase(object):
     """
     Class that contains helper methods for both filter.series as well as plugins that configure it,
-    such as all_series, series_premiere and import_series.
+    such as all_series, series_premiere and configure_series.
     """
 
     @property
@@ -800,20 +826,7 @@ class FilterSeriesBase(object):
         return task.config['series']
 
 
-@event('manager.execute.started')
-def remove_old_tasks(manager):
-    """Unmark series from tasks which have been deleted"""
-    session = Session()
-    try:
-        deleted = (session.query(SeriesTask).filter(not_(SeriesTask.name.in_(manager.config['tasks']))).
-                   delete(synchronize_session=False))
-        if deleted:
-            session.commit()
-    finally:
-        session.close()
-
-
-class FilterSeries(SeriesDatabase, FilterSeriesBase):
+class FilterSeries(FilterSeriesBase):
     """
     Intelligent filter for tv-series.
 
@@ -852,12 +865,9 @@ class FilterSeries(SeriesDatabase, FilterSeriesBase):
         }
 
     def __init__(self):
-        self.backlog = None
-
-    def on_process_start(self, task):
         try:
-            self.backlog = get_plugin_by_name('backlog').instance
-        except DependencyError:
+            self.backlog = plugin.get_plugin_by_name('backlog')
+        except plugin.DependencyError:
             log.warning('Unable utilize backlog plugin, episodes may slip trough timeframe')
 
     def auto_exact(self, config):
@@ -879,9 +889,9 @@ class FilterSeries(SeriesDatabase, FilterSeriesBase):
                         series_config['exact'] = True
 
     # Run after metainfo_quality and before metainfo_series
-    @priority(125)
-    def on_task_metainfo(self, task):
-        config = self.prepare_config(task.config.get('series', {}))
+    @plugin.priority(125)
+    def on_task_metainfo(self, task, config):
+        config = self.prepare_config(config)
         self.auto_exact(config)
         for series_item in config:
             series_name, series_config = series_item.items()[0]
@@ -891,13 +901,13 @@ class FilterSeries(SeriesDatabase, FilterSeriesBase):
             took = time.clock() - start_time
             log.trace('parsing %s took %s', series_name, took)
 
-    def on_task_filter(self, task):
+    def on_task_filter(self, task, config):
         """Filter series"""
         # Parsing was done in metainfo phase, create the dicts to pass to process_series from the task entries
         # key: series episode identifier ie. S01E02
         # value: seriesparser
 
-        config = self.prepare_config(task.config.get('series', {}))
+        config = self.prepare_config(config)
         found_series = {}
         for entry in task.entries:
             if entry.get('series_name') and entry.get('series_id') is not None and entry.get('series_parser'):
@@ -923,7 +933,7 @@ class FilterSeries(SeriesDatabase, FilterSeriesBase):
             series_entries = {}
             for entry in found_series[series_name]:
                 # store found episodes into database and save reference for later use
-                releases = self.store(task.session, entry['series_parser'], series=db_series)
+                releases = store_parser(task.session, entry['series_parser'], series=db_series)
                 entry['series_releases'] = releases
                 series_entries.setdefault(releases[0].episode, []).append(entry)
 
@@ -937,7 +947,7 @@ class FilterSeries(SeriesDatabase, FilterSeriesBase):
 
                 # accept info from set: and place into the entry
                 if 'set' in series_config:
-                    set = get_plugin_by_name('set')
+                    set = plugin.get_plugin_by_name('set')
                     set.instance.modify(entry, series_config.get('set'))
 
             # If we didn't find any episodes for this series, continue
@@ -950,7 +960,7 @@ class FilterSeries(SeriesDatabase, FilterSeriesBase):
                 db_series.identified_by = series_config['identified_by']
             # if series doesn't have identified_by flag already set, calculate one now that new eps are added to db
             if not db_series.identified_by or db_series.identified_by == 'auto':
-                db_series.identified_by = self.auto_identified_by(db_series)
+                db_series.identified_by = auto_identified_by(db_series)
                 log.debug('identified_by set to \'%s\' based on series history', db_series.identified_by)
 
             log.trace('series_name: %s series_config: %s', series_name, series_config)
@@ -1038,7 +1048,7 @@ class FilterSeries(SeriesDatabase, FilterSeriesBase):
 
             # accept info from set: and place into the entry
             if 'set' in config:
-                set = get_plugin_by_name('set')
+                set = plugin.get_plugin_by_name('set')
                 set.instance.modify(entry, config.get('set'))
 
     def process_series(self, task, series_entries, config):
@@ -1136,7 +1146,7 @@ class FilterSeries(SeriesDatabase, FilterSeriesBase):
 
             # episode advancement. used only with season and sequence based series
             if ep.identified_by in ['ep', 'sequence']:
-                if task.manager.options.disable_advancement:
+                if task.options.disable_advancement:
                     log.debug('episode advancement disabled')
                 else:
                     log.debug('-' * 20 + ' episode advancement -->')
@@ -1259,7 +1269,9 @@ class FilterSeries(SeriesDatabase, FilterSeriesBase):
     def process_episode_advancement(self, episode, entries, grace):
         """Rejects all episodes that are too old or new (advancement), return True when this happens."""
 
-        latest = self.get_latest_download(episode.series)
+        latest = get_latest_download(episode.series)
+        if episode.series.begin and episode.series.begin > latest:
+            latest = episode.series.begin
         log.debug('latest download: %s' % latest)
         log.debug('current: %s' % episode)
 
@@ -1308,7 +1320,7 @@ class FilterSeries(SeriesDatabase, FilterSeriesBase):
         expires = first_seen + timeframe
         log.debug('timeframe: %s, first_seen: %s, expires: %s', timeframe, first_seen, expires)
 
-        stop = task.manager.options.stop_waiting.lower() == episode.series.name.lower()
+        stop = task.options.stop_waiting.lower() == episode.series.name.lower()
         if expires <= datetime.now() or stop:
             # Expire timeframe, accept anything
             log.info('Timeframe expired, releasing quality restriction.')
@@ -1326,7 +1338,7 @@ class FilterSeries(SeriesDatabase, FilterSeriesBase):
 
             # add best entry to backlog (backlog is able to handle duplicate adds)
             if self.backlog:
-                self.backlog.add_backlog(task, best)
+                self.backlog.instance.add_backlog(task, best)
             return True
 
     def process_qualities(self, config, entries, downloaded):
@@ -1372,7 +1384,7 @@ class FilterSeries(SeriesDatabase, FilterSeriesBase):
                 still_needed = [req for req in still_needed if not req.allows(quality)]
         return bool(downloaded_qualities)
 
-    def on_task_exit(self, task):
+    def on_task_exit(self, task, config):
         """Learn succeeded episodes"""
         log.debug('on_task_exit')
         for entry in task.accepted:
@@ -1387,7 +1399,7 @@ class FilterSeries(SeriesDatabase, FilterSeriesBase):
 class SeriesDBManager(FilterSeriesBase):
     """Update in the database with series info from the config"""
 
-    @priority(0)
+    @plugin.priority(0)
     def on_task_start(self, task, config):
         if not task.config_modified:
             return
@@ -1418,14 +1430,20 @@ class SeriesDBManager(FilterSeriesBase):
                 try:
                     set_series_begin(db_series, series_config['begin'])
                 except ValueError as e:
-                    raise PluginError(e)
+                    raise plugin.PluginError(e)
 
 
-# Register plugin
-register_plugin(FilterSeries, 'series')
-# This is a builtin so that it can update the database for tasks that may have had series plugin removed
-register_plugin(SeriesDBManager, 'series_db', builtin=True, api_ver=2)
-register_parser_option('--stop-waiting', action='store', dest='stop_waiting', default='',
-                       metavar='NAME', help='Stop timeframe for a given series.')
-register_parser_option('--disable-advancement', action='store_true', dest='disable_advancement', default=False,
-                       help='Disable episode advancement for this run.')
+@event('plugin.register')
+def register_plugin():
+    plugin.register(FilterSeries, 'series', api_ver=2)
+    # This is a builtin so that it can update the database for tasks that may have had series plugin removed
+    plugin.register(SeriesDBManager, 'series_db', builtin=True, api_ver=2)
+
+
+@event('options.register')
+def register_parser_arguments():
+    exec_parser = options.get_parser('execute')
+    exec_parser.add_argument('--stop-waiting', action='store', dest='stop_waiting', default='',
+                             metavar='NAME', help='stop timeframe for a given series')
+    exec_parser.add_argument('--disable-advancement', action='store_true', dest='disable_advancement', default=False,
+                             help='disable episode advancement for this run')
