@@ -4,11 +4,12 @@ import csv
 import re
 from cgi import parse_header
 
+from flexget import plugin
+from flexget.event import event
 from flexget.utils import requests
 from flexget.utils.imdb import make_url
 from flexget.utils.cached_input import cached
 from flexget.utils.tools import decode_html
-from flexget.plugin import register_plugin, PluginError
 from flexget.entry import Entry
 from flexget.utils.soup import get_soup
 
@@ -57,7 +58,7 @@ class ImdbList(object):
                 # Now we do the actual login with appropriate parameters
                 r = sess.post('https://secure.imdb.com/register-imdb/login', data=params, raise_status=False)
             except requests.RequestException as e:
-                raise PluginError('Unable to login to imdb: %s' % e.message)
+                raise plugin.PluginError('Unable to login to imdb: %s' % e.message)
 
             # IMDb redirects us upon a successful login.
             # removed - doesn't happen always?
@@ -78,10 +79,10 @@ class ImdbList(object):
                 if re.match(USER_ID_RE, user_id):
                     config['user_id'] = user_id
                 else:
-                    raise PluginError('Couldn\'t figure out user_id, please configure it manually.')
+                    raise plugin.PluginError('Couldn\'t figure out user_id, please configure it manually.')
 
         if not 'user_id' in config:
-            raise PluginError('Configuration option `user_id` required.')
+            raise plugin.PluginError('Configuration option `user_id` required.')
 
         log.verbose('Retrieving list %s ...' % config['list'])
 
@@ -94,11 +95,11 @@ class ImdbList(object):
             mime_type = parse_header(opener.headers['content-type'])[0]
             log.debug('mime_type: %s' % mime_type)
             if mime_type != 'text/csv':
-                raise PluginError('Didn\'t get CSV export as response. Probably specified list `%s` does not exist.'
-                                  % config['list'])
+                raise plugin.PluginError('Didn\'t get CSV export as response. Probably specified list `%s` '
+                                         'does not exist.' % config['list'])
             csv_rows = csv.reader(opener.iter_lines())
         except requests.RequestException as e:
-            raise PluginError('Unable to get imdb list: %s' % e.message)
+            raise plugin.PluginError('Unable to get imdb list: %s' % e.message)
 
         # Create an Entry for each movie in the list
         entries = []
@@ -114,4 +115,6 @@ class ImdbList(object):
         return entries
 
 
-register_plugin(ImdbList, 'imdb_list', api_ver=2)
+@event('plugin.register')
+def register_plugin():
+    plugin.register(ImdbList, 'imdb_list', api_ver=2)
