@@ -13,7 +13,8 @@ from httplib import BadStatusLine
 
 from requests import RequestException
 
-from flexget.plugin import register_plugin, register_parser_option, PluginWarning, PluginError
+from flexget import options, plugin
+from flexget.event import event
 from flexget.utils.tools import decode_html
 from flexget.utils.template import RenderError
 from flexget.utils.pathscrub import pathscrub
@@ -186,7 +187,7 @@ class PluginDownload(object):
         :return: String error, if failed.
         """
         try:
-            if task.manager.options.test:
+            if task.options.test:
                 log.info('Would download: %s' % entry['title'])
             else:
                 if not task.manager.unit_test:
@@ -276,7 +277,7 @@ class PluginDownload(object):
 
         # check for write-access
         if not os.access(tmp_path, os.W_OK):
-            raise PluginError('Not allowed to write to temp directory `%s`' % tmp_path)
+            raise plugin.PluginError('Not allowed to write to temp directory `%s`' % tmp_path)
 
         # download and write data into a temp file
         tmp_dir = tempfile.mkdtemp(dir=tmp_path)
@@ -368,7 +369,7 @@ class PluginDownload(object):
         for entry in task.accepted:
             try:
                 self.output(task, entry, config)
-            except PluginWarning as e:
+            except plugin.PluginWarning as e:
                 entry.fail()
                 log.error('Plugin error while writing: %s' % e)
             except Exception as e:
@@ -382,19 +383,19 @@ class PluginDownload(object):
             PluginError if operation fails
         """
 
-        if 'file' not in entry and not task.manager.options.test:
+        if 'file' not in entry and not task.options.test:
             log.debug('file missing, entry: %s' % entry)
-            raise PluginError('Entry `%s` has no temp file associated with' % entry['title'])
+            raise plugin.PluginError('Entry `%s` has no temp file associated with' % entry['title'])
 
         try:
             # use path from entry if has one, otherwise use from download definition parameter
             path = entry.get('path', config.get('path'))
             if not isinstance(path, basestring):
-                raise PluginError('Invalid `path` in entry `%s`' % entry['title'])
+                raise plugin.PluginError('Invalid `path` in entry `%s`' % entry['title'])
 
             # override path from command line parameter
-            if task.manager.options.dl_path:
-                path = task.manager.options.dl_path
+            if task.options.dl_path:
+                path = task.options.dl_path
 
             # expand variables in path
             try:
@@ -407,7 +408,7 @@ class PluginDownload(object):
             path = pathscrub(path)
 
             # If we are in test mode, report and return
-            if task.manager.options.test:
+            if task.options.test:
                 log.info('Would write `%s` to `%s`' % (entry['title'], path))
                 # Set a fake location, so the exec plugin can do string replacement during --test #1015
                 entry['output'] = os.path.join(path, 'TEST_MODE_NO_OUTPUT')
@@ -419,12 +420,12 @@ class PluginDownload(object):
                 try:
                     os.makedirs(path)
                 except:
-                    raise PluginError('Cannot create path %s' % path, log)
+                    raise plugin.PluginError('Cannot create path %s' % path, log)
 
             # check that temp file is present
             if not os.path.exists(entry['file']):
                 log.debug('entry: %s' % entry)
-                raise PluginWarning('Downloaded temp file `%s` doesn\'t exist!?' % entry['file'])
+                raise plugin.PluginWarning('Downloaded temp file `%s` doesn\'t exist!?' % entry['file'])
 
             # if we still don't have a filename, try making one from title (last resort)
             if not entry.get('filename'):
@@ -472,7 +473,7 @@ class PluginDownload(object):
                     # ignore permission errors, see ticket #555
                     import errno
                     if not os.path.exists(destfile):
-                        raise PluginError('Unable to write %s' % destfile)
+                        raise plugin.PluginError('Unable to write %s' % destfile)
                     if err.errno != errno.EPERM:
                         raise
 
@@ -503,6 +504,13 @@ class PluginDownload(object):
         for entry in task.entries + task.rejected + task.failed:
             self.cleanup_temp_file(entry)
 
-register_plugin(PluginDownload, 'download', api_ver=2)
-register_parser_option('--dl-path', action='store', dest='dl_path', default=False,
-                       metavar='PATH', help='Override path for download plugin. Applies to all executed tasks.')
+
+@event('plugin.register')
+def register_plugin():
+    plugin.register(PluginDownload, 'download', api_ver=2)
+
+
+@event('options.register')
+def register_parser_arguments():
+    options.get_parser('execute').add_argument('--dl-path', dest='dl_path', default=False, metavar='PATH',
+                                               help='override path for download plugin, applies to all executed tasks')
