@@ -1,28 +1,32 @@
 from __future__ import unicode_literals, division, absolute_import
 import logging
 from Queue import Empty
-from flask import render_template, request, flash
-from flask import Module, escape, jsonify
-from flexget.ui.webui import register_plugin, manager, executor
-from flexget.ui.executor import BufferQueue
 
-execute = Module(__name__, url_prefix='/execute')
+from flask import render_template, request, flash
+from flask import Blueprint, escape, jsonify
+
+from flexget.options import CoreArgumentParser, RaiseErrorArgumentParser
+from flexget.ui.webui import register_plugin
+from flexget.scheduler import BufferQueue
+
+execute = Blueprint('execute', __name__)
 
 log = logging.getLogger('ui.execute')
 
 bufferqueue = BufferQueue()
-
+exec_parser = RaiseErrorArgumentParser(parents=[CoreArgumentParser().get_subparser('execute')])
 
 @execute.route('/', methods=['POST', 'GET'])
 def index():
-    context = {'progress': manager.parser.get_help().split('\n')}
+    context = {'progress': exec_parser.format_help().split('\n')}
     if request.method == 'POST':
-        parser = manager.parser.parse_args(request.form.get('options', ''))
-        if manager.parser.error_msg:
-            flash(escape(manager.parser.error_msg), 'error')
+        try:
+            options = exec_parser.parse_args(request.form.get('options', ''))
+        except ValueError as e:
+            flash(escape(e.message), 'error')
             context['options'] = request.form['options']
         else:
-            executor.execute(parsed_options=parser, output=bufferqueue)
+            executor.execute(options=options, output=bufferqueue)
             context['execute_progress'] = True
             context['progress'] = progress(as_list=True)
 
